==> kgdata-4.1.0-cp310-cp310-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: kgdata/wikidata/datasets/entity_types_and_degrees.py <==
from __future__ import annotations

import math
from dataclasses import dataclass
from functools import partial
from random import randrange
from typing import Iterable, Optional

import orjson
from kgdata.dataset import Dataset
from kgdata.misc.resource import Record
from kgdata.models.ont_class import OntologyClass
from kgdata.spark import are_records_unique, does_result_dir_exist, get_spark_context
from kgdata.wikidata.config import WikidataDirCfg
from kgdata.wikidata.datasets.class_count import class_count
from kgdata.wikidata.datasets.entity_all_types import EntityAllTypes, entity_all_types
from kgdata.wikidata.datasets.entity_degrees import EntityDegree, entity_degrees


@dataclass
class EntityTypeAndDegree(Record):
    id: str
    types: dict[str, int]
    indegree: int
    outdegree: int

    wikipedia_indegree: Optional[int] = None
    wikipedia_outdegree: Optional[int] = None


def entity_types_and_degrees(lang: str = "en") -> Dataset[EntityTypeAndDegree]:
    cfg = WikidataDirCfg.get_instance()

    if not does_result_dir_exist(cfg.entity_types_and_degrees):
        (
            entity_all_types(lang)
            .get_rdd()
            .map(lambda e: (e.id, e))
            .join(entity_degrees(lang).get_rdd().map(lambda e: (e.id, e)))
            .map(merge_type_degree)
            .map(EntityTypeAndDegree.ser)
            .coalesce(128)
            .saveAsTextFile(
                str(cfg.entity_types_and_degrees),
                compressionCodecClass="org.apache.hadoop.io.compress.GzipCodec",
            )
        )

    return Dataset(
        cfg.entity_types_and_degrees / "*.gz", deserialize=EntityTypeAndDegree.deser
    )


def merge_type_degree(
    tup: tuple[str, tuple[EntityAllTypes, EntityDegree]]
) -> EntityTypeAndDegree:
    return EntityTypeAndDegree(
        id=tup[0],
        types=tup[1][0].types,
        indegree=tup[1][1].indegree,
        outdegree=tup[1][1].outdegree,
        wikipedia_indegree=tup[1][1].wikipedia_indegree,
        wikipedia_outdegree=tup[1][1].wikipedia_outdegree,
    )

==> kgdata-7.0.6-cp310-cp310-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: kgdata/wikidata/datasets/entity_types_and_degrees.py <==
from __future__ import annotations

from dataclasses import dataclass
from functools import lru_cache
from typing import Optional

from kgdata.dataset import Dataset
from kgdata.misc.resource import Record
from kgdata.wikidata.config import WikidataDirCfg
from kgdata.wikidata.datasets.entity_all_types import EntityAllTypes, entity_all_types
from kgdata.wikidata.datasets.entity_degrees import EntityDegree, entity_degrees


@dataclass
class EntityTypeAndDegree(Record):
    id: str
    # mapping from type to distance of the correct types
    # zero is the type of that entity, 1 is the parent type, 2 is grand parent, etc.
    types: dict[str, int]
    indegree: int
    outdegree: int

    wikipedia_indegree: Optional[int] = None
    wikipedia_outdegree: Optional[int] = None


@lru_cache()
def entity_types_and_degrees() -> Dataset[EntityTypeAndDegree]:
    cfg = WikidataDirCfg.get_instance()
    ds = Dataset(
        cfg.entity_types_and_degrees / "*.gz",
        deserialize=EntityTypeAndDegree.deser,
        name="entity-types-and-degrees",
        dependencies=[entity_all_types(), entity_degrees()],
    )
    if not ds.has_complete_data():
        (
            entity_all_types()
            .get_extended_rdd()
            .map(lambda e: (e.id, e))
            .join(entity_degrees().get_extended_rdd().map(lambda e: (e.id, e)))
            .map(merge_type_degree)
            .map(EntityTypeAndDegree.ser)
            .save_like_dataset(ds, auto_coalesce=True, shuffle=True)
        )

    return ds


def merge_type_degree(
    tup: tuple[str, tuple[EntityAllTypes, EntityDegree]]
) -> EntityTypeAndDegree:
    return EntityTypeAndDegree(
        id=tup[0],
        types=tup[1][0].types,
        indegree=tup[1][1].indegree,
        outdegree=tup[1][1].outdegree,
        wikipedia_indegree=tup[1][1].wikipedia_indegree,
        wikipedia_outdegree=tup[1][1].wikipedia_outdegree,
    )
